using System;

/// @author  Vesa Lappalainen, Antti-Jussi Lakanen
/// @version 2025
/// 
/// <summary>
/// Demo 6, Bonus 2-3.
/// Lukujen vertailuja tietyllä tarkkuudella
/// </summary>
public class B2_3_Samat
{
    private const double EPS = 10e-7;


    /// <summary>
    /// 
    /// </summary>
    public static void Main()
    {
        double a = 7.1001;
        double b = 7.1002;
        double c = 7.2002;
        bool lahella = Samat(a, b, 0.01);
        if (lahella) Console.WriteLine("Ovat melkein samoja");
        if (!Samat(a, b)) Console.WriteLine("Ovat eri suuria");
        if (!Samat(a, c, 0.01)) Console.WriteLine("Ovat eri suuria");
        if (Samat(a, c, 0.2)) Console.WriteLine("Ovat sinnepäin");
    }

    public static bool Samat(double a, double b)
    {
        return Samat(a, b, 10e-7);
    }

    /// <summary>
    /// Verrataan kahta lukua, ovatko ne enintään annetun luvun
    /// päässä toisistaan.
    /// </summary>
    /// <param name="a">toinen luku</param>
    /// <param name="b">toinen luku</param>
    /// <param name="eps">millä tarkkuudella verrataan</param>
    /// <returns>true jos ovat riittävän lähellä toisiaan</returns>
    /// <example>
    /// <pre name="test">
    ///  Samat(7.1, 7.2, 0.01)   === false;
    ///  Samat(7.1, 7.2, 0.2)    === true;
    ///  Samat(-7.1, -7.2, 0.01) === false;
    ///  Samat(-7.1, -7.2, 0.2)  === true;
    ///  Samat(0.1, -0.1, 0.3)   === true;
    ///  Samat(0.1, -0.1, 0.1)   === false;
    ///  Samat(0.1, 0.01) === false;
    ///  Samat(0.00000001, 0.00000015) === true;
    ///  Samat(1.00000001, 1.00000015) === true;
    ///  Samat(1.0000001, 1.0000015)  === false;
    ///  Samat( 10000001, 10000015)   === false;
    ///  Samat(100000001, 100000015)   === false;
    /// </pre>
    /// </example>
    public static bool Samat(double a, double b, double suurinEtaisyys)
    {
        if (Etaisyys(a, b) <= suurinEtaisyys) return true;
        else return false;
    }

    /// <summary>
    /// Kahden luvun välinen etäisyys
    /// </summary>
    /// <param name="a">Luku 1</param>
    /// <param name="b">luku 2</param>
    /// <returns>Etäisyys</returns>
    /// <example>
    /// <pre name="test">
    /// Etaisyys(5, 3) === 2;
    /// Etaisyys(3, 5) === 2;
    /// Etaisyys(-3, -5) === 2;
    /// Etaisyys(0, 0) === 0;
    /// Etaisyys(-1, 1) === 2;
    /// Etaisyys(-1, 2) === 3;
    /// </pre>
    /// </example>
    public static double Etaisyys(double a, double b)
    {
        double erotus = a - b;
        double itseisarvo = Itseisarvo(erotus);
        return itseisarvo;

        // TAI:
        // return Itseisarvo(a - b);
        // TAI: 
        // return Itseisarvo(erotus);
    }

    /// <summary>
    /// Luvun itseisarvo
    /// </summary>
    /// <param name="luku">Luku</param>
    /// <returns>Itseisarvo</returns>
    public static double Itseisarvo(double luku)
    {
        if (luku < 0)
        {
            return luku * (-1);
        }

        else return luku;
    }

    /// <summary>
    /// Tehtävä B3. Verrataan ovatko kaksi lukua suhteellisesti lähellä toisiaan.
    /// </summary>
    /// <param name="a">ensimmäinen verrattava</param>
    /// <param name="b">toinen verrattava</param>
    /// <param name="eps">suhdeluku, joka otetaan ensimmäisestä
    /// verrattavasta luvusta. Toisen luvun etäisyyttä
    /// ensimmäiseen lukuun verrataan tässä suhteessa ensimmäiseen lukuun.
    /// <returns>tosi jos luvut suhteellisesti lähellä toisiaan</returns>
    /// <example>
    /// <pre name="test">
    ///  SuhtSamat(0.10, 0.12, 0.1) === false
    ///  SuhtSamat(0.10, 0.11, 0.1) === true
    ///  SuhtSamat(0.11, 0.10, 0.1) === true
    ///  SuhtSamat(1.0, 1.2, 0.1)   === false
    ///  SuhtSamat(1.0, 1.1, 0.1)   === true
    ///  SuhtSamat(10, 12, 0.1)     === false
    ///  SuhtSamat(10, 11, 0.1)     === true
    ///  SuhtSamat(1000, 1200, 0.1) === false
    ///  SuhtSamat(1000, 1100, 0.1) === true
    ///  SuhtSamat(900, 1000, 0.1)  === true
    /// </pre>
    /// </example>
    public static bool SuhtSamat(double a, double b, double eps = EPS)
    {
        double amax = Math.Max(Math.Abs(a), Math.Abs(b));
        return Math.Abs(a - b) <= eps * amax;
        // tai:
        // return Keskiluku.Etaisyys(a, b) <= eps * amax;
    }

}